"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const HttpClient_1 = __importDefault(require("./HttpClient"));
class Client {
    constructor(config) {
        this.config = config;
        this.httpClient = new HttpClient_1.default({
            apiKey: this.config.apiKey,
            baseUrl: `${constants_1.API_URL_BASE}`,
        });
    }
    //
    // Tenant methods
    //
    getTenant(tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/tenants/${tenantId}`,
                });
            }
            catch (e) {
                console.log("Error getting tenant");
                throw e;
            }
        });
    }
    createTenant(tenant = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: "/v1/tenants",
                    data: tenant,
                });
            }
            catch (e) {
                console.log("Error creating tenant");
                throw e;
            }
        });
    }
    updateTenant(tenantId, tenant) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.put({
                    url: `/v1/tenants/${tenantId}`,
                    data: tenant,
                });
            }
            catch (e) {
                console.log("Error updating tenant");
                throw e;
            }
        });
    }
    deleteTenant(tenantId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.delete({
                    url: `/v1/tenants/${tenantId}`,
                });
            }
            catch (e) {
                console.log("Error deleting tenant");
                throw e;
            }
        });
    }
    listTenants(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: "/v1/tenants",
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing tenants");
                throw e;
            }
        });
    }
    listTenantsForUser(userId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/users/${userId}/tenants`,
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing tenants for user");
                throw e;
            }
        });
    }
    //
    // User methods
    //
    getUser(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/users/${userId}`,
                });
            }
            catch (e) {
                console.log("Error getting user");
                throw e;
            }
        });
    }
    createUser(user = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: "/v1/users",
                    data: user,
                });
            }
            catch (e) {
                console.log("Error creating user");
                throw e;
            }
        });
    }
    updateUser(userId, user) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.put({
                    url: `/v1/users/${userId}`,
                    data: user,
                });
            }
            catch (e) {
                console.log("Error updating user");
                throw e;
            }
        });
    }
    deleteUser(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.delete({
                    url: `/v1/users/${userId}`,
                });
            }
            catch (e) {
                console.log("Error deleting user");
                throw e;
            }
        });
    }
    listUsers(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: "/v1/users",
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing users");
                throw e;
            }
        });
    }
    listUsersForTenant(tenantId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/tenants/${tenantId}/users`,
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing users for tenant");
                throw e;
            }
        });
    }
    addUserToTenant(tenantId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: `/v1/tenants/${tenantId}/users/${userId}`,
                });
            }
            catch (e) {
                console.log("Error adding user to tenant");
                throw e;
            }
        });
    }
    removeUserFromTenant(tenantId, userId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.delete({
                    url: `/v1/tenants/${tenantId}/users/${userId}`,
                });
            }
            catch (e) {
                console.log("Error removing user from tenant");
                throw e;
            }
        });
    }
    //
    // Role methods
    //
    getRole(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/roles/${roleId}`,
                });
            }
            catch (e) {
                console.log("Error getting role");
                throw e;
            }
        });
    }
    createRole(role) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: "/v1/roles",
                    data: role,
                });
            }
            catch (e) {
                console.log("Error creating role in Warrant");
                throw e;
            }
        });
    }
    deleteRole(roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete({
                    url: `/v1/roles/${roleId}`,
                });
            }
            catch (e) {
                console.log("Error deleting role");
                throw e;
            }
        });
    }
    listRoles(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: "/v1/roles",
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing roles");
                throw e;
            }
        });
    }
    listRolesForUser(userId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/users/${userId}/roles`,
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing roles for user");
                throw e;
            }
        });
    }
    assignRoleToUser(userId, roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: `/v1/users/${userId}/roles/${roleId}`,
                });
            }
            catch (e) {
                console.log("Error assigning role to user");
                throw e;
            }
        });
    }
    removeRoleFromUser(userId, roleId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete({
                    url: `/users/${userId}/roles/${roleId}`,
                });
            }
            catch (e) {
                console.log("Error removing role from user");
                throw e;
            }
        });
    }
    //
    // Permission methods
    //
    getPermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/permissions/${permissionId}`,
                });
            }
            catch (e) {
                console.log("Error getting permission");
                throw e;
            }
        });
    }
    createPermission(permission) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: "/v1/permissions",
                    data: permission,
                });
            }
            catch (e) {
                console.log("Error creating permission");
                throw e;
            }
        });
    }
    deletePermission(permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete({
                    url: `/v1/permissions/${permissionId}`,
                });
            }
            catch (e) {
                console.log("Error deleting permission");
                throw e;
            }
        });
    }
    listPermissions(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: "/v1/permissions",
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing permissions");
                throw e;
            }
        });
    }
    listPermissionsForUser(userId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/users/${userId}/permissions`,
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing permissions for user");
                throw e;
            }
        });
    }
    listPermissionsForRole(roleId, listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: `/v1/roles/${roleId}/permissions`,
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing permissions for role");
                throw e;
            }
        });
    }
    assignPermissionToUser(userId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: `/v1/users/${userId}/permissions/${permissionId}`,
                });
            }
            catch (e) {
                console.log("Error assigning permission to user");
                throw e;
            }
        });
    }
    removePermissionFromUser(userId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete({
                    url: `/v1/users/${userId}/permissions/${permissionId}`,
                });
            }
            catch (e) {
                console.log("Error removing permission from user");
                throw e;
            }
        });
    }
    assignPermissionToRole(roleId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: `/v1/roles/${roleId}/permissions/${permissionId}`,
                });
            }
            catch (e) {
                console.log("Error assigning permission to role");
                throw e;
            }
        });
    }
    removePermissionFromRole(roleId, permissionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpClient.delete({
                    url: `/v1/roles/${roleId}/permissions/${permissionId}`,
                });
            }
            catch (e) {
                console.log("Error removing permission from role");
                throw e;
            }
        });
    }
    //
    // Warrant methods
    //
    createWarrant(warrant) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.post({
                    url: "/v1/warrants",
                    data: warrant,
                });
            }
            catch (e) {
                console.log("Error creating warrant");
                throw e;
            }
        });
    }
    deleteWarrant(warrant) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.delete({
                    url: "/v1/warrants",
                    data: warrant,
                });
            }
            catch (e) {
                console.log("Error deleting warrant");
                throw e;
            }
        });
    }
    listWarrants(listOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.httpClient.get({
                    url: "/warrants",
                    params: listOptions,
                });
            }
            catch (e) {
                console.log("Error listing warrants");
                throw e;
            }
        });
    }
    /**
     * Creates an authorization session in Warrant for the specified userId and returns a session token that can be used to
     * make authorization requests to the Warrant API scoped to the specified userId and tenantId.
     *
     * @param session A session object containing the userId, redirectUrl, and optional tenantId for which the authorization session should be created.
     * @returns A session token that can be passed to any of the Warrant client-side SDKs to allow the SDK to make client-side authorization checks for the specified user.
     */
    createAuthorizationSession(session) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const sess = yield this.httpClient.post({
                    url: "/v1/sessions",
                    data: Object.assign(Object.assign({}, session), { type: "sess" }),
                });
                return sess.token;
            }
            catch (e) {
                console.log("Error creating authorization session for user");
                throw e;
            }
        });
    }
    /**
     * Creates a self-service session in Warrant for the user with the specified userId and returns a session token that can be used to
     * make authorized requests to the Warrant API scoped to the specified userId and tenantId.
     *
     * @param session A session object containing the userId, redirectUrl, and optional tenantId for which the self service session should be created.
     * @returns A url pointing to the self-service dashboard that will allow the specified user to make changes to the roles and permissions of users in their tenant.
     */
    createSelfServiceSession(session, redirectUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const sess = yield this.httpClient.post({
                    url: "/v1/sessions",
                    data: Object.assign(Object.assign({}, session), { type: "ssdash" }),
                });
                return `${constants_1.SELF_SERVICE_DASH_URL_BASE}/${sess.token}?redirectUrl=${redirectUrl}`;
            }
            catch (e) {
                console.log("Error creating self-service session for user");
                throw e;
            }
        });
    }
    /**
     * Checks against the warrants defined for your application to determine if the given `userId` has the given
     * `relation` to the given `objectId` of type `objectType`. Returns true if the the relation exists and false otherwise.
     * @param objectType The type of the object for which access is being checked.
     * @param objectId The id of the object for which access is being checked.
     * @param relation The relation to the object for which access is being checked.
     * @param userId The userId for which access is being checked.
     * @returns True if the `userId` has the given `relation` to `objectId` of type `objectType`, and false otherwise.
     */
    /**
     * Checks against the warrants defined for your application to determine if the given `userId` has the given
     * `relation` to the given `objectId` of type `objectType`. Returns true if the the relation exists and false otherwise.
     * @param options access check options for this authorization request, including the objectType, objectId, relation, and subject
     * for which access is being checked.
     * @returns True if the `subject` has the given `relation` to `objectId` of type `objectType`, and false otherwise.
     */
    isAuthorized(warrantCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.config.authorizeEndpoint) {
                return this.edgeAuthorize(warrantCheck);
            }
            return this.authorize(warrantCheck);
        });
    }
    //
    // Authorize methods
    //
    authorize(warrantCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post({
                    url: "/v2/authorize",
                    data: warrantCheck,
                });
                return response.code === 200;
            }
            catch (e) {
                console.log("Error performing access check");
                throw e;
            }
        });
    }
    edgeAuthorize(warrantCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.httpClient.post({
                    baseUrl: this.config.authorizeEndpoint,
                    url: "/v2/authorize",
                    data: warrantCheck,
                });
                return response.code === 200;
            }
            catch (e) {
                if (e.code === "cache_not_ready") {
                    console.log("Edge cache not ready. Re-routing access check to api.warrant.dev");
                    return this.authorize(warrantCheck);
                }
                console.log("Error performing access check");
                throw e;
            }
        });
    }
    hasPermission(permissionCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.isAuthorized({
                warrants: [{
                        objectType: "permission",
                        objectId: permissionCheck.permissionId,
                        relation: "member",
                        subject: {
                            objectType: "user",
                            objectId: permissionCheck.userId,
                        },
                    }],
                consistentRead: permissionCheck.consistentRead,
                debug: permissionCheck.debug,
            });
        });
    }
}
exports.default = Client;
